@@ -2,9 +2,9 @@
  * S-Lang bindings for the expat library.
  * This was tested with expat versions 1.95.8 and 2.0.
  * 
- * $Id: expat-module.c,v 1.3 2007/06/23 13:15:00 paul Exp paul $
+ * $Id: expat-module.c,v 1.6 2008/06/01 12:02:34 paul Exp paul $
  * 
- * Copyright (c) 2006, 2007 Paul Boekholt.
+ * Copyright (c) 2006-2008 Paul Boekholt.
  * Released under the terms of the GNU GPL (version 2 or later).
  */
 #include <slang.h>
@@ -14,9 +14,9 @@
 SLANG_MODULE(expat);
 
 #define MODULE_MAJOR_VERSION	0
-#define MODULE_MINOR_VERSION	3
+#define MODULE_MINOR_VERSION	5
 #define MODULE_PATCH_LEVEL	0
-static char *Module_Version_String = "0.3.0";
+static char *Module_Version_String = "0.5.0";
 #define MODULE_VERSION_NUMBER	\
    (MODULE_MAJOR_VERSION*10000+MODULE_MINOR_VERSION*100+MODULE_PATCH_LEVEL)
 
@@ -169,7 +169,7 @@ static const Expat_Exception_Table_Type Expat_Exception_Table [] =
      { XML_ERROR_NONE, 0, 0, 0}
 };
 
-static int translate_error(error_code)
+static int translate_error (int error_code)
 {
    const Expat_Exception_Table_Type *b;
    if (error_code == XML_ERROR_NONE) return 0;
@@ -185,14 +185,14 @@ static int translate_error(error_code)
    else return *(b->errcode_ptr);
 }
 
-static int check_error (XML_Parser p)
+static int check_error (XML_Parser p, SLang_MMT_Type *mmt)
 {
    int error_code;
    int error;
    error_code = XML_GetErrorCode(p);
    if ((error = translate_error(error_code)))
      {
-	SLang_verror (error, (char *) XML_ErrorString(error_code));
+	SLerr_throw(error,  (char *) XML_ErrorString(error_code), Expat_Type_Id, &mmt);
 	return 1;
      }
    return 0;
@@ -220,7 +220,6 @@ static void slxml_new(void)
      {
 	SLang_free_mmt (mmt);
 	(void) SLang_push_null();
-	XML_ParserFree(p);
      }
 }
 
@@ -243,7 +242,6 @@ static void slxml_new_ns(const char *sep)
      {
 	SLang_free_mmt (mmt);
 	(void) SLang_push_null();
-	XML_ParserFree(p);
      }
 }
 
@@ -256,13 +254,12 @@ static void slxml_parse(const char *s, int *isfinal)
    SLang_MMT_Type *mmt;
    if (NULL == (mmt = SLang_pop_mmt (Expat_Type_Id)))
      {
-	SLang_free_mmt (mmt);
 	return;
      }
    parser = SLang_object_from_mmt (mmt);
    if (XML_STATUS_OK != XML_Parse(parser->p, s, strlen(s), *isfinal))
      {
-	check_error(parser->p);
+	check_error(parser->p, mmt);
      }
    SLang_free_mmt (mmt);
 }
@@ -273,30 +270,28 @@ static void slxml_stop_parser(int *resumable)
    SLang_MMT_Type *mmt;
    if (NULL == (mmt = SLang_pop_mmt (Expat_Type_Id)))
      {
-	SLang_free_mmt (mmt);
 	return;
      }
    parser = SLang_object_from_mmt (mmt);
    if (XML_STATUS_OK != XML_StopParser(parser->p, *resumable))
      {
-	check_error(parser->p);
+	check_error(parser->p, mmt);
      }
    SLang_free_mmt (mmt);
 }
 
-static void slxml_resume_parser()
+static void slxml_resume_parser (void)
 {
    expat_type *parser;
    SLang_MMT_Type *mmt;
    if (NULL == (mmt = SLang_pop_mmt (Expat_Type_Id)))
      {
-	SLang_free_mmt (mmt);
 	return;
      }
    parser = SLang_object_from_mmt (mmt);
    if (XML_STATUS_OK != XML_ResumeParser(parser->p))
      {
-	check_error(parser->p);
+	check_error(parser->p, mmt);
      }
    SLang_free_mmt (mmt);
 }
@@ -313,7 +308,6 @@ static void slxml_setuserdata(void)
    
    if (NULL == (mmt = SLang_pop_mmt (Expat_Type_Id)))
      {
-	SLang_free_mmt (mmt);
 	return;
      }
    parser = SLang_object_from_mmt (mmt);
@@ -330,7 +324,6 @@ static void slxml_getuserdata(void)
    
    if (NULL == (mmt = SLang_pop_mmt (Expat_Type_Id)))
      {
-	SLang_free_mmt (mmt);
 	SLang_push_null();
 	return;
      }
@@ -426,7 +419,7 @@ static int pop_function_or_null(SLang_Name_Type **func)
    return 1;
 }
 
-static void slxml_set_start_element_handler()
+static void slxml_set_start_element_handler (void)
 {
    SLang_MMT_Type *mmt;
    expat_type *parser;
@@ -436,7 +429,6 @@ static void slxml_set_start_element_handler()
    
    if (NULL == (mmt = SLang_pop_mmt (Expat_Type_Id)))
      {
-	SLang_free_mmt (mmt);
 	return;
      }
    
@@ -466,7 +458,7 @@ static void endelementhandler(void *userData, const char *name)
    }
 }
 
-static void slxml_set_end_element_handler()
+static void slxml_set_end_element_handler (void)
 {
    SLang_MMT_Type *mmt;
    expat_type *parser;
@@ -476,7 +468,6 @@ static void slxml_set_end_element_handler()
    
    if (NULL == (mmt = SLang_pop_mmt (Expat_Type_Id)))
      {
-	SLang_free_mmt (mmt);
 	return;
      }
    parser = SLang_object_from_mmt (mmt);
@@ -508,7 +499,7 @@ static void characterdatahandler(void *userData, const char *s, const int len)
    SLang_free_slstring(str);
 }
 
-static void slxml_set_character_data_handler()
+static void slxml_set_character_data_handler (void)
 {
    SLang_MMT_Type *mmt;
    expat_type *parser;
@@ -518,7 +509,6 @@ static void slxml_set_character_data_handler()
    
    if (NULL == (mmt = SLang_pop_mmt (Expat_Type_Id)))
      {
-	SLang_free_mmt (mmt);
 	return;
      }
    
@@ -549,7 +539,7 @@ static void startnamespacedeclhandler(void *userData, const char *prefix, const
    }
 }
 
-static void slxml_set_start_namespace_decl_handler()
+static void slxml_set_start_namespace_decl_handler (void)
 {
    SLang_MMT_Type *mmt;
    expat_type *parser;
@@ -559,7 +549,6 @@ static void slxml_set_start_namespace_decl_handler()
    
    if (NULL == (mmt = SLang_pop_mmt (Expat_Type_Id)))
      {
-	SLang_free_mmt (mmt);
 	return;
      }
    
@@ -589,7 +578,7 @@ static void endnamespacedeclhandler(void *userData, const char *prefix)
    }
 }
 
-static void slxml_set_end_namespace_decl_handler()
+static void slxml_set_end_namespace_decl_handler (void)
 {
    SLang_MMT_Type *mmt;
    expat_type *parser;
@@ -599,7 +588,6 @@ static void slxml_set_end_namespace_decl_handler()
    
    if (NULL == (mmt = SLang_pop_mmt (Expat_Type_Id)))
      {
-	SLang_free_mmt (mmt);
 	return;
      }
    
@@ -632,7 +620,7 @@ static void defaulthandler(void *userData, const char *s, const int len)
    SLang_free_slstring(str);
 }
 
-static void slxml_set_default_handler()
+static void slxml_set_default_handler (void)
 {
    SLang_MMT_Type *mmt;
    expat_type *parser;
@@ -642,7 +630,6 @@ static void slxml_set_default_handler()
    
    if (NULL == (mmt = SLang_pop_mmt (Expat_Type_Id)))
      {
-	SLang_free_mmt (mmt);
 	return;
      }
    
